@@ -37,14 +37,13 @@
 #' Computational statistics & data analysis, 44(4), 571-586.
 #'
 #' @keywords internal
-calcStandardErrors <- function(C, D)
-{
+calcStandardErrors <- function(C,
+                               D) {
   ## !!! NOT CHANGE THE LINE OF CODE BELOW !!!
   ## It adds extra zeros ("fill-ins") to matrix C, needed
   ## to calculate the Partial Derivatives of Cholesky, not equal to zero.
   C = C + 0 * spam::crossprod.spam(D)
   cholC <- chol(C)
-
   p <- cholC@pivot
   tD <- t(D)
   tDp <- tD[p, ]
@@ -52,4 +51,3 @@ calcStandardErrors <- function(C, D)
   se <- sqrt(x)
   return(se)
 }
-
