@@ -33,6 +33,8 @@ List construct_ADchol_Rcpp(SEXP U,
   IntegerVector colpointers = Rcpp::clone<Rcpp::IntegerVector>(obj_spam.slot("colpointers"));
   IntegerVector colindices = Rcpp::clone<Rcpp::IntegerVector>(obj_spam.slot("colindices"));
   IntegerVector pivot = Rcpp::clone<Rcpp::IntegerVector>(obj_spam.slot("pivot"));
+  IntegerVector invpivot = Rcpp::clone<Rcpp::IntegerVector>(obj_spam.slot("invpivot"));
+
   IntegerVector Dim = Rcpp::clone<Rcpp::IntegerVector>(obj_spam.slot("dimension"));
 
   // copy not really needed or used ...
@@ -44,6 +46,7 @@ List construct_ADchol_Rcpp(SEXP U,
   transf2C(colpointers);
   transf2C(colindices);
   transf2C(pivot);
+  transf2C(invpivot);
 
   const int size = rowpointers[rowpointers.size()-1];
 
@@ -93,6 +96,8 @@ List construct_ADchol_Rcpp(SEXP U,
   L["colpointers"] = rowpointers;
   L["rowpointers"] = colpointers;
   L["rowindices"] =  colindices;
+  L["pivot"] = pivot;
+  L["invpivot"] = invpivot;
   L["entries"] = entries;
   L["ADentries"] = ADentries;
   L["P"] = P_matrix;
@@ -454,6 +459,98 @@ double logdet(SEXP arg, NumericVector lambda)
 }
 
 
+NumericVector forwardCholesky(
+    const NumericVector& L,
+    const NumericVector& b,
+    const IntegerVector& supernodes,
+    const IntegerVector& rowpointers,
+    const IntegerVector& colpointers,
+    const IntegerVector& rowindices,
+    const IntegerVector& pivot,
+    const IntegerVector& invpivot)
+{
+  const int Nsupernodes = supernodes.size()-1;
+  const int N = colpointers.size() - 1;
+  NumericVector x(N);
+  NumericVector Pb(N);  // permutation of P
+  NumericVector sum(N); // sum for each column
+  for (int i=0;i<N;i++)
+  {
+    Pb[i] = b[pivot[i]];
+  }
+  for (int J=0; J<Nsupernodes;J++)
+  {
+    int s = rowpointers[J];
+    for (int j=supernodes[J]; j<supernodes[J+1]; j++)
+    {
+      const double x_j = (Pb[j]-sum[j])/L[colpointers[j]];
+      x[j] = x_j;
+      // the non-diagonal elements of column j
+      for (int ndx = colpointers[j]+1, k=s+1; ndx < colpointers[j+1]; ndx++)
+      {
+        int i = rowindices[k++];
+        sum[i] += L[ndx]*x_j;
+      }
+      s++;
+    }
+  }
+
+  NumericVector xP(N); // inverse permutation
+  for (int i=0;i<N;i++) {
+    xP[i] = x[invpivot[i]];
+  }
+  return xP;
+
+}
+
+
+NumericVector backwardCholesky(
+    const NumericVector& L,
+    const NumericVector& b,
+    const IntegerVector& supernodes,
+    const IntegerVector& rowpointers,
+    const IntegerVector& colpointers,
+    const IntegerVector& rowindices,
+    const IntegerVector& pivot,
+    const IntegerVector& invpivot)
+{
+  const int Nsupernodes = supernodes.size()-1;
+  const int N = colpointers.size() - 1;
+  NumericVector x(N);
+  NumericVector Pb(N);  // permutation of P
+  NumericVector sum(N); // sum for each column
+  for (int i=0;i<N;i++)
+  {
+    Pb[i] = b[pivot[i]];
+  }
+  for (int J=Nsupernodes-1; J>=0;J--)
+  {
+    int NodeSz = supernodes[J+1] - supernodes[J];
+    int s = rowpointers[J]+(NodeSz-1);
+    for (int j=supernodes[J+1]-1; j>=supernodes[J]; j--)
+    {
+      //Rcout << "column " << j << endl;
+      double alpha = L[colpointers[j]];
+      double x_j = Pb[j];
+      // the non-diagonal elements of column j
+      for (int ndx = colpointers[j]+1, k=s+1; ndx < colpointers[j+1]; ndx++)
+      {
+        int i = rowindices[k++];
+        x_j -= L[ndx]*x[i];
+      }
+      x[j] = x_j/alpha;
+      s--;
+    }
+  }
+
+  NumericVector xP(N); // inverse permutation
+  for (int i=0;i<N;i++) {
+    xP[i] = x[invpivot[i]];
+  }
+  return xP;
+}
+
+
 //' Calculate the partial derivatives of log-determinant.
 //'
 //' This function calculates the partial derivatives of the the log-determinant in an
@@ -479,7 +576,8 @@ double logdet(SEXP arg, NumericVector lambda)
 //' @keywords internal
 //'
 // [[Rcpp::export]]
-NumericVector dlogdet(SEXP ADobj, NumericVector theta)
+NumericVector dlogdet(SEXP ADobj, NumericVector theta,
+                      Nullable<NumericVector> b_ = R_NilValue)
 {
   Rcpp::S4 obj(ADobj);
   IntegerVector supernodes = obj.slot("supernodes");
@@ -541,6 +639,17 @@ NumericVector dlogdet(SEXP ADobj, NumericVector theta)
 
   gradient.attr("logdet") = logDet;
 
+  if (b_.isNotNull()) {
+    NumericVector b(b_);        // casting to underlying type NumericVector
+    IntegerVector pivot = obj.slot("pivot");
+    IntegerVector invpivot = obj.slot("invpivot");
+
+    NumericVector z= forwardCholesky(L, b, supernodes, rowpointers,
+                             colpointers, rowindices, pivot, invpivot);
+    NumericVector x = backwardCholesky(L, z, supernodes, rowpointers,
+                     colpointers, rowindices, pivot, invpivot);
+    gradient.attr("x.coef") = x;
+  }
   return gradient;
 }
 
@@ -637,9 +746,64 @@ NumericVector diagXCinvXt(SEXP cholC, SEXP transposeX)
   return H;
 }
 
-/*
 
-Not used, just to show the structure of the sparse cholesky matrix with supernodes.
+// [[Rcpp::export]]
+NumericVector ForwardCholesky(SEXP cholC, NumericVector& b)
+{
+  Rcpp::S4 obj(cholC);
+  // We use transpose for calculating Automated Differentiation.
+  IntegerVector supernodes = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("supernodes"));
+  IntegerVector colpointers = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("rowpointers"));
+  IntegerVector rowpointers = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("colpointers"));
+  IntegerVector rowindices = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("colindices"));
+  IntegerVector pivot = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("pivot"));
+  IntegerVector invpivot = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("invpivot"));
+
+  NumericVector L = Rcpp::clone<Rcpp::NumericVector>(obj.slot("entries"));
+
+  // C using indices starting at 0:
+  transf2C(supernodes);
+  transf2C(colpointers);
+  transf2C(rowpointers);
+  transf2C(rowindices);
+  transf2C(pivot);
+  transf2C(invpivot);
+
+  return forwardCholesky(L, b, supernodes, rowpointers,
+                      colpointers, rowindices, pivot, invpivot);
+
+}
+
+
+// [[Rcpp::export]]
+NumericVector BackwardCholesky(SEXP cholC, NumericVector& b)
+{
+  Rcpp::S4 obj(cholC);
+  // We use transpose for calculating Automated Differentiation.
+  IntegerVector supernodes = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("supernodes"));
+  IntegerVector colpointers = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("rowpointers"));
+  IntegerVector rowpointers = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("colpointers"));
+  IntegerVector rowindices = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("colindices"));
+  IntegerVector pivot = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("pivot"));
+  IntegerVector invpivot = Rcpp::clone<Rcpp::IntegerVector>(obj.slot("invpivot"));
+
+  NumericVector L = Rcpp::clone<Rcpp::NumericVector>(obj.slot("entries"));
+
+  // C using indices starting at 0:
+  transf2C(supernodes);
+  transf2C(colpointers);
+  transf2C(rowpointers);
+  transf2C(rowindices);
+  transf2C(pivot);
+  transf2C(invpivot);
+
+  return backwardCholesky(L, b, supernodes, rowpointers,
+                         colpointers, rowindices, pivot, invpivot);
+}
+
+//Not used, just to show the structure of the sparse cholesky matrix with supernodes.
+
+/*
 
 // [[Rcpp::export]]
 NumericMatrix PrintCholesky(SEXP cholC)
